/*	TOP-SURF: a visual words toolkit
	Copyright (C) 2010 LIACS Media Lab, Leiden University,
	                   Bart Thomee (bthomee@liacs.nl),
					   Erwin M. Bakker (erwin@liacs.nl)	and
					   Michael S. Lew (mlew@liacs.nl).

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    See http://www.gnu.org/licenses/gpl.html for the full license.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

	In addition, this work is covered under the Creative Commons
	Attribution license version 3.
    See http://creativecommons.org/licenses/by/3.0/ for the full license.
*/

// import the topsurf library
#include "topsurf/api.h"
#if defined WIN32 || defined _WIN32 || defined WIN64 || defined _WIN64
	#ifdef _WIN64
		#ifdef _DEBUG
			#pragma comment(lib, "topsurf_x64d.lib")
		#else
			#pragma comment(lib, "topsurf_x64.lib")
		#endif
	#else
		#ifdef _DEBUG
			#pragma comment(lib, "topsurf_x32d.lib")
		#else
			#pragma comment(lib, "topsurf_x32.lib")
		#endif
	#endif
#endif

#include <stdarg.h>
#include <stdio.h>

// safe print string to file and immediately flush it
void SAFE_FLUSHPRINT(FILE *file, const char *format, ...)
{
	va_list	argptr;
	va_start(argptr, format);
	vfprintf(file, format, argptr);
	va_end(argptr);
	fflush(file);
}

// wait for user to type a character
#if defined WIN32 || defined _WIN32 || defined WIN64 || defined _WIN64
#include <conio.h>
#else
#include <termios.h>
#include <unistd.h>
int _getch()
{
	struct termios oldt, newt;
	int ch;
	tcgetattr(STDIN_FILENO, &oldt);
	newt = oldt;
	newt.c_lflag &= ~(ICANON | ECHO);
	tcsetattr(STDIN_FILENO, TCSANOW, &newt);
	ch = getchar();
	tcsetattr(STDIN_FILENO, TCSANOW, &oldt);
	return ch;
}
#endif

// image names for comparison
#define IMAGE_NAME1		"compare1.jpg"
#define IMAGE_NAME2		"compare2.jpg"
#define IMAGE_NAME3		"compare3.jpg"
#define IMAGE_NAME4		"compare4.jpg"

int main(int argc, char *argv[])
{
	// initialize TOP-SURF, with default options of 100 elements and resizing the image to
	// 256x256 pixels. the "dict" subdirectory is used for loading the dictionary
	SAFE_FLUSHPRINT(stdout, "Please wait while the TOP-SURF library initializes...\n");
	if (!TopSurf_Initialize(256, 100))
		return 1;
	if (!TopSurf_LoadDictionary("dict"))
	{
		TopSurf_Terminate();
		return 1;
	}
	// extract the descriptors of four images, of which image one is the 'original' image,
	// image two is a brightness-adjusted version of the original image, image three
	// is the original image but with black borders around it, and image four is a
	// completely different image
	SAFE_FLUSHPRINT(stdout, "Extracting descriptors...\n");
	TOPSURF_DESCRIPTOR td1, td2, td3, td4;
	if (!TopSurf_ExtractDescriptor(IMAGE_NAME1, td1) ||
		!TopSurf_ExtractDescriptor(IMAGE_NAME2, td2) ||
		!TopSurf_ExtractDescriptor(IMAGE_NAME3, td3) ||
		!TopSurf_ExtractDescriptor(IMAGE_NAME4, td4))
	{
		SAFE_FLUSHPRINT(stdout, "Could not extract descriptors\n");
		TopSurf_ReleaseDescriptor(td1);
		TopSurf_ReleaseDescriptor(td2);
		TopSurf_ReleaseDescriptor(td3);
		TopSurf_ReleaseDescriptor(td4);
		TopSurf_Terminate();
		return 1;
	}
	// compare the original image (td1) with the completely different image (td2) and the
	// highly similar image (td3), using both cosine and absolute similarity measures
	SAFE_FLUSHPRINT(stdout, "Comparing descriptors using cosine normalized difference...\n");
	float distance = TopSurf_CompareDescriptors(td1, td2, TOPSURF_COSINE);
	SAFE_FLUSHPRINT(stdout, "  the distance between image %s and %s is %.2f\n", IMAGE_NAME1, IMAGE_NAME2, distance);
	distance = TopSurf_CompareDescriptors(td1, td3, TOPSURF_COSINE);
	SAFE_FLUSHPRINT(stdout, "  the distance between image %s and %s is %.2f\n", IMAGE_NAME1, IMAGE_NAME3, distance);
	distance = TopSurf_CompareDescriptors(td1, td4, TOPSURF_COSINE);
	SAFE_FLUSHPRINT(stdout, "  the distance between image %s and %s is %.2f\n", IMAGE_NAME1, IMAGE_NAME4, distance);
	SAFE_FLUSHPRINT(stdout, "Comparing descriptors using absolute difference...\n");
	distance = TopSurf_CompareDescriptors(td1, td2, TOPSURF_ABSOLUTE);
	SAFE_FLUSHPRINT(stdout, "  the distance between image %s and %s is %.2f\n", IMAGE_NAME1, IMAGE_NAME2, distance);
	distance = TopSurf_CompareDescriptors(td1, td3, TOPSURF_ABSOLUTE);
	SAFE_FLUSHPRINT(stdout, "  the distance between image %s and %s is %.2f\n", IMAGE_NAME1, IMAGE_NAME3, distance);
	distance = TopSurf_CompareDescriptors(td1, td4, TOPSURF_ABSOLUTE);
	SAFE_FLUSHPRINT(stdout, "  the distance between image %s and %s is %.2f\n", IMAGE_NAME1, IMAGE_NAME4, distance);
	// release resources
	SAFE_FLUSHPRINT(stdout, "Please wait while the TOP-SURF library terminates...\n");
	TopSurf_ReleaseDescriptor(td1);
	TopSurf_ReleaseDescriptor(td2);
	TopSurf_ReleaseDescriptor(td3);
	TopSurf_ReleaseDescriptor(td4);
	TopSurf_Terminate();
	SAFE_FLUSHPRINT(stdout, "Press any key to continue...\n");
	_getch();
	return 0;
}